@@ -1,4 +1,4 @@
-# $Id: Perl.pm,v 1.126 2009/02/02 01:18:27 turnstep Exp $
+# $Id: Perl.pm,v 1.71 2001/05/02 23:16:56 btrott Exp $
 
 package Net::SSH::Perl;
 use strict;
@@ -14,7 +14,6 @@ use vars qw( $VERSION $CONFIG $HOSTNAME );
 $CONFIG = {};
 
 use Socket;
-use IO::Socket;
 use Fcntl;
 use Symbol;
 use Carp qw( croak );
@@ -23,7 +22,7 @@ eval {
     $HOSTNAME = hostname();
 };
 
-$VERSION = '1.34.1';
+$VERSION = "1.01";
 
 sub VERSION { $VERSION }
 
@@ -55,35 +54,34 @@ sub set_protocol {
     $ssh->_proto_init;
 }
 
-use vars qw( @COMPAT );
-@COMPAT = (
-  [  '^OpenSSH[-_]2\.[012]' => SSH_COMPAT_OLD_SESSIONID,   ],
-  [  'MindTerm'             => 0,                          ],
-  [  '^2\.1\.0 '            => SSH_COMPAT_BUG_SIGBLOB |
-                               SSH_COMPAT_BUG_HMAC |
-                               SSH_COMPAT_OLD_SESSIONID,   ],
-  [  '^2\.0\.'              => SSH_COMPAT_BUG_SIGBLOB |
-                               SSH_COMPAT_BUG_HMAC |
-                               SSH_COMPAT_OLD_SESSIONID |
-                               SSH_COMPAT_BUG_PUBKEYAUTH |
-                               SSH_COMPAT_BUG_X11FWD,      ],
-  [  '^2\.[23]\.0 '         => SSH_COMPAT_BUG_HMAC,        ],
-  [  '^2\.[2-9]\.'          => 0,                          ],
-  [  '^2\.4$'               => SSH_COMPAT_OLD_SESSIONID,   ],
-  [  '^3\.0 SecureCRT'      => SSH_COMPAT_OLD_SESSIONID,   ],
-  [  '^1\.7 SecureFX'       => SSH_COMPAT_OLD_SESSIONID,   ],
-  [  '^2\.'                 => SSH_COMPAT_BUG_HMAC,        ],
+use vars qw( %COMPAT );
+%COMPAT = (
+    '^OpenSSH[-_]2\.[012]' => SSH_COMPAT_OLD_SESSIONID,
+    'MindTerm'             => 0,
+    '^2\.1\.0 '            => SSH_COMPAT_BUG_SIGBLOB |
+                              SSH_COMPAT_BUG_HMAC |
+                              SSH_COMPAT_OLD_SESSIONID,
+    '^2\.0\.'              => SSH_COMPAT_BUG_SIGBLOB |
+                              SSH_COMPAT_BUG_HMAC |
+                              SSH_COMPAT_OLD_SESSIONID |
+                              SSH_COMPAT_BUG_PUBKEYAUTH |
+                              SSH_COMPAT_BUG_X11FWD,
+    '^2\.[23]\.0 '         => SSH_COMPAT_BUG_HMAC,
+    '^2\.[2-9]\.'          => 0,
+    '^2\.4$'               => SSH_COMPAT_OLD_SESSIONID,
+    '^3\.0 SecureCRT'      => SSH_COMPAT_OLD_SESSIONID,
+    '^1\.7 SecureFX'       => SSH_COMPAT_OLD_SESSIONID,
+    '^2\.'                 => SSH_COMPAT_BUG_HMAC,
 );
 
 sub _compat_init {
     my $ssh = shift;
     my($version) = @_;
     $ssh->{datafellows} = 0;
-    for my $rec (@COMPAT) {
-        my($re, $mask) = @$rec[0, 1];
+    for my $re (keys %COMPAT) {
         if ($version =~ /$re/) {
-            $ssh->debug("Compat match: '$version' matches pattern '$re'.");
-            $ssh->{datafellows} = $mask;
+            $ssh->debug("Compat match: $version matches pattern '$re'.");
+            $ssh->{datafellows} = $COMPAT{$re};
             return;
         }
     }
@@ -95,12 +93,6 @@ sub version_string { }
 sub client_version_string { $_[0]->{client_version_string} }
 sub server_version_string { $_[0]->{server_version_string} }
 
-sub _current_user {
-    my $user;
-    eval { $user = scalar getpwuid $> };
-    return $user;
-}
-
 sub _init {
     my $ssh = shift;
 
@@ -110,10 +102,6 @@ sub _init {
 
     my $directives = delete $arg{options} || [];
 
-    if (my $proto = delete $arg{protocol}) {
-        push @$directives, "Protocol $proto";
-    }
-
     my $cfg = Net::SSH::Perl::Config->new($ssh->{host}, %arg);
     $ssh->{config} = $cfg;
 
@@ -133,8 +121,7 @@ sub _init {
         $ssh->{host} = $real_host;
     }
 
-    my $user = _current_user();
-    if ($user && $user eq "root" &&
+    if (scalar getpwuid($<) eq "root" &&
       !defined $ssh->{config}->get('privileged')) {
         $ssh->{config}->set('privileged', 1);
     }
@@ -170,7 +157,7 @@ sub ssh {
     my($host, @cmd) = @_;
     my($user);
     ($host, $user) = $host =~ m!(.+)@(.+)! ?
-       ($2, $1) : ($host, _current_user());
+       ($2, $1) : ($host, scalar getpwuid($<));
     my $ssh = __PACKAGE__->new($host, %$CONFIG);
     $ssh->login($user);
     my($out, $err, $exit) = $ssh->cmd(join ' ', @cmd);
@@ -199,7 +186,9 @@ sub _connect {
     my $rport = $ssh->{config}->get('port') || 'ssh';
     if ($rport =~ /\D/) {
         my @serv = getservbyname(my $serv = $rport, 'tcp');
-        $rport = $serv[2] || 22;
+        $rport = $serv[2];
+        croak "Can't map service name '$serv' to port number"
+            unless defined $rport;
     }
     $ssh->debug("Connecting to $ssh->{host}, port $rport.");
     connect($sock, sockaddr_in($rport, $raddr))
@@ -210,7 +199,7 @@ sub _connect {
     $ssh->{session}{sock} = $sock;
     $ssh->_exchange_identification;
 
-    defined($sock->blocking(0))
+    fcntl($sock, F_SETFL, O_NONBLOCK)
         or die "Can't set socket non-blocking: $!";
 
     $ssh->debug("Connection established.");
@@ -219,31 +208,26 @@ sub _connect {
 sub _create_socket {
     my $ssh = shift;
     my $sock = gensym;
-
-	my ($p,$end,$delta) = (0,1,1); # normally we use whatever port we can get
-   	   ($p,$end,$delta) = (1023,512,-1) if $ssh->{config}->get('privileged');
-
-	# allow an explicit bind address
-    my $addr = $ssh->{config}->get('bind_address');
-	$addr = inet_aton($addr) if $addr;
-	($p,$end,$delta) = (10000,65535,1) if $addr and not $p;
-	$addr ||= INADDR_ANY;
-
-    for(; $p != $end; $p += $delta) {
-        socket($sock, AF_INET, SOCK_STREAM, getprotobyname('tcp') || 0) ||
-            croak "Net::SSH: Can't create socket: $!";
-        last if not $p or bind($sock, sockaddr_in($p,$addr));
-        if ($! =~ /Address already in use/i) {
-            close($sock) or warn qq{Could not close socket: $!\n};
-            next;
+    if ($ssh->{config}->get('privileged')) {
+        my $p;
+        my $proto = getprotobyname('tcp');
+        for ($p = 1023; $p > 512; $p--) {
+            socket($sock, AF_INET, SOCK_STREAM, $proto) ||
+                croak "Net::SSH: Can't create socket: $!";
+            last if bind($sock, sockaddr_in($p, INADDR_ANY));
+            if ($! =~ /Address already in use/i) {
+                close($sock);
+                next;
+            }
+            croak "Net::SSH: Can't bind socket to port $p: $!";
         }
-        croak "Net::SSH: Can't bind socket to port $p: $!";
+        $ssh->debug("Allocated local port $p.");
+        $ssh->{config}->set('localport', $p);
+    }
+    else {
+        socket($sock, AF_INET, SOCK_STREAM, 0) ||
+            croak "Net::SSH: Can't create socket: $!";
     }
-	if($p) {
-		$ssh->debug("Allocated local port $p.");
-		$ssh->{config}->set('localport', $p);
-	}
-
     $sock;
 }
 
@@ -261,7 +245,7 @@ sub _exchange_identification {
     my $ssh = shift;
     my $sock = $ssh->{session}{sock};
     my $remote_id = <$sock>;
-    ($ssh->{server_version_string} = $remote_id) =~ s/\cM?\n$//;
+    $ssh->{server_version_string} = substr $remote_id, 0, -1;
     my($remote_major, $remote_minor, $remote_version) = $remote_id =~
         /^SSH-(\d+)\.(\d+)-([^\n]+)\n$/;
     $ssh->debug("Remote protocol version $remote_major.$remote_minor, remote software version $remote_version");
@@ -314,7 +298,7 @@ sub login {
     my($user, $pass) = @_;
     if (!defined $ssh->{config}->get('user')) {
         $ssh->{config}->set('user',
-            defined $user ? $user : _current_user());
+            defined $user ? $user : scalar getpwuid($<));
     }
     if (!defined $pass && exists $CONFIG->{ssh_password}) {
         $pass = $CONFIG->{ssh_password};
@@ -330,14 +314,14 @@ sub shell { }
 sub incoming_data {
     my $ssh = shift;
     if (!exists $ssh->{session}{incoming_data}) {
-        $ssh->{session}{incoming_data} = Net::SSH::Perl::Buffer->new( MP => $ssh->protocol == PROTOCOL_SSH2 ? 'SSH2' : 'SSH1' );
+        $ssh->{session}{incoming_data} = Net::SSH::Perl::Buffer->new;
     }
     $ssh->{session}{incoming_data};
 }
 
 sub session_id {
     my $ssh = shift;
-    $ssh->{session}{id} = shift if @_ and not defined $ssh->{session}{id};
+    $ssh->{session}{id} = shift if @_;
     $ssh->{session}{id};
 }
 
@@ -392,9 +376,8 @@ Net::SSH::Perl - Perl client Interface to SSH
 
 =head1 DESCRIPTION
 
-I<Net::SSH::Perl> is an all-Perl module implementing an SSH
-(Secure Shell) client. It is compatible with both the SSH-1 and
-SSH-2 protocols.
+I<Net::SSH::Perl> is an all-Perl module implementing an SSH client.
+It is compatible with both the SSH1 and SSH2 protocols.
 
 I<Net::SSH::Perl> enables you to simply and securely execute commands
 on remote machines, and receive the STDOUT, STDERR, and exit status
@@ -430,18 +413,10 @@ also be fully compatible with the "official" SSH implementation. If
 you find an SSH2 implementation that is not compatible with
 I<Net::SSH::Perl>, please let me know (email address down in
 I<AUTHOR & COPYRIGHTS>); it turns out that some SSH2 implementations
-have subtle differences from others. 3DES (C<3des-cbc>), Blowfish
-(C<blowfish-cbc>), and RC4 (C<arcfour>) ciphers are currently
-supported for SSH2 encryption, and integrity checking is performed
-by either the C<hmac-sha1> or C<hmac-md5> algorithms. Compression, if
-requested, is limited to Zlib. Supported server host key algorithms
-are C<ssh-dss> (the default) and C<ssh-rsa> (requires I<Crypt::RSA>);
-supported SSH2 public key authentication algorithms are the same.
-
-If you're looking for SFTP support, take a look at I<Net::SFTP>,
-which provides a full-featured Perl implementation of SFTP, and
-sits on top of I<Net::SSH::Perl>. SFTP requires the usage of the
-SSH2 protocol.
+have subtle differences from others. 3DES and Blowfish ciphers are
+currently supported for SSH2 encryption, and integrity checking is
+performed by either the hmac-sha1 or hmac-md5 algorithms. Compression,
+if requested, is currently limited to Zlib.
 
 =head1 BASIC USAGE
 
@@ -460,7 +435,7 @@ I<new> accepts the following named parameters in I<%params>:
 
 The protocol you wish to use for the connection: should be either
 C<2>, C<1>, C<'1,2'> or C<'2,1'>. The first two say, quite simply,
-"only use this version of the protocol" (SSH-2 or SSH-1, respectively).
+"only use this version of the protocol" (SSH2 or SSH1, respectively).
 The latter two specify that either protocol can be used, but that
 one protocol (the first in the comma-separated list) is preferred
 over the other.
@@ -472,7 +447,7 @@ the second will be used. (Presumably your server will support at
 least one of the two protocols. :)
 
 The default value is C<'1,2'>, for compatibility with OpenSSH; this
-means that the client will use SSH-1 if the server supports SSH-1.
+means that the client will use SSH1 if the server supports SSH1.
 Of course, you can always override this using a user/global
 configuration file, or through using this constructor argument.
 
@@ -481,13 +456,13 @@ configuration file, or through using this constructor argument.
 Specifies the name of the encryption cipher that you wish to
 use for this connection. This must be one of the supported
 ciphers; specifying an unsupported cipher will give you an error
-when you enter algorithm negotiation (in either SSH-1 or SSH-2).
+when you enter algorithm negotiation (in either SSH1 or SSH2).
 
-In SSH-1, the supported cipher names are I<IDEA>, I<DES>, I<DES3>,
-and I<Blowfish>; in SSH-2, the supported ciphers are I<arcfour>,
+In SSH1, the supported cipher names are I<IDEA>, I<DES>, I<DES3>,
+and I<Blowfish>; in SSH2, the supported ciphers are I<arcfour>,
 I<blowfish-cbc>, and I<3des-cbc>.
 
-The default SSH-1 cipher is I<IDEA>; the default SSH-2 cipher is
+The default SSH1 cipher is I<IDEA>; the default SSH2 cipher is
 I<3des-cbc>.
 
 =item * ciphers
@@ -495,9 +470,9 @@ I<3des-cbc>.
 Like I<cipher>, this is a method of setting the cipher you wish to
 use for a particular SSH connection; but this corresponds to the
 I<Ciphers> configuration option, where I<cipher> corresponds to
-I<Cipher>. This also applies only in SSH-2.
+I<Cipher>. This also applies only in SSH2.
 
-This should be a comma-separated list of SSH-2 cipher names; the list
+This should be a comma-separated list of SSH2 cipher names; the list
 of cipher names is listed above in I<cipher>.
 
 This defaults to I<3des-cbc,blowfish-cbc,arcfour>.
@@ -520,7 +495,7 @@ Defaults to false.
 
 =item * interactive
 
-Set to a true value if you're using I<Net::SSH::Perl> interactively.
+Set to a true value if you're using I<Net::SSH> interactively.
 This is used in determining whether or not to display password
 prompts, for example. It's basically the inverse of the
 I<BatchMode> parameter in ssh configuration.
@@ -571,8 +546,8 @@ Specifies the compression level to use if compression is enabled
 I<compression_level> arguments to set the level; providing only
 this argument will not turn on encryption).
 
-This setting is only applicable to SSH-1; the compression level for
-SSH-2 Zlib compression is always set to 6.
+This setting is only applicable to SSH1; the compression level for
+SSH2 Zlib compression is always set to 6.
 
 The default value is 6.
 
@@ -602,7 +577,7 @@ directives in the format used in the config file. For example:
 
 =back
 
-=head2 $ssh->login([ $user [, $password [, $suppress_shell ] ] ])
+=head2 $ssh->login([ $user [, $password ] ])
 
 Sets the username and password to be used when authenticating
 with the I<sshd> daemon. The username I<$user> is required for
@@ -611,16 +586,10 @@ remote server), but if you don't supply it the username of the
 user executing the program is used.
 
 The password I<$password> is needed only for password
-authentication (it's not used for passphrases on encrypted
-RSA/DSA identity files, though perhaps it should be). And if you're
-running in an interactive session and you've not provided a
-password, you'll be prompted for one.
-
-By default, Net::SSH::Perl will open a channel with a shell
-on it. This is usually what you want. If you are tunneling
-another protocol over SSH, however, you may want to
-prevent this behavior.  Passing a true value in I<$suppress_shell>
-will prevent the shell channel from being opened (SSH2 only).
+authentication (it's not used for RSA passphrase authentication,
+though perhaps it should be). And if you're running in an
+interactive session and you've not provided a password, you'll
+be prompted for one.
 
 =head2 ($out, $err, $exit) = $ssh->cmd($cmd, [ $stdin ])
 
@@ -631,7 +600,7 @@ command.
 If I<$stdin> is provided, it's supplied to the remote command
 I<$cmd> on standard input.
 
-NOTE: the SSH-1 protocol does not support running multiple commands
+NOTE: the SSH1 protocol does not support running multiple commands
 per connection, unless those commands are chained together so that
 the remote shell can evaluate them. Because of this, a new socket
 connection is created each time you call I<cmd>, and disposed of
@@ -647,7 +616,7 @@ will actually connect to the I<sshd> on the first invocation of
 I<cmd>, then disconnect; then connect again on the second
 invocation of I<cmd>, then disconnect again.
 
-Note that this does I<not> apply to the SSH-2 protocol. SSH-2 fully
+Note that this does I<not> apply to the SSH2 protocol. SSH2 fully
 supports running more than one command over the same connection.
 
 =head2 $ssh->shell
@@ -680,16 +649,11 @@ block, in case your program exits prior to reaching that line.
 If you need an example, take a look at F<eg/pssh>, which
 uses almost this exact code to implement an ssh shell.
 
-=head2 $ssh->register_handler($packet_type, $subref [, @args ])
+=head2 $ssh->register_handler($packet_type, $subref)
 
 Registers an anonymous subroutine handler I<$subref> to handle
 packets of type I<$packet_type> during the client loop. The
-subroutine will be called when packets of type I<$packet_type>
-are received, and in addition to the standard arguments (see
-below), will receive any additional arguments in I<@args>, if
-specified.
-
-The client loop is entered after the client has sent a command
+client loop is entered after the client has sent a command
 to the remote server, and after any STDIN data has been sent;
 it consists of reading packets from the server (STDOUT
 packets, STDERR packets, etc.) until the server sends the exit
@@ -707,28 +671,24 @@ by calling the I<register_handler> method and setting up handlers
 to be called at specific times.
 
 The behavior of the I<register_handler> method differs between
-the I<Net::SSH::Perl> SSH-1 and SSH-2 implementations. This is so
+the I<Net::SSH::Perl> SSH1 and SSH2 implementations. This is so
 because of the differences between the protocols (all 
-client-server communications in SSH-2 go through the channel
+client-server communications in SSH2 go through the channel
 mechanism, which means that input packets are processed
 differently).
 
 =over 4
 
-=item * SSH-1 Protocol
+=item * SSH1 Protocol
 
-In the SSH-1 protocol, you should call I<register_handler> with two
+In the SSH1 protocol, you should call I<register_handler> with two
 arguments: a packet type I<$packet_type> and a subroutine reference
 I<$subref>. Your subroutine will receive as arguments the
 I<Net::SSH::Perl::SSH1> object (with an open connection to the
 ssh3), and a I<Net::SSH::Perl::Packet> object, which represents the
-packet read from the server. It will also receive any additional
-arguments I<@args> that you pass to I<register_handler>; this can
-be used to give your callback functions access to some of your
-otherwise private variables, if desired. I<$packet_type> should be
-an integer constant; you can import the list of constants into your
-namespace by explicitly loading the I<Net::SSH::Perl::Constants>
-module:
+packet read from the server. I<$packet_type> should be an integer
+constant; you can import the list of constants into your namespace
+by explicitly loading the I<Net::SSH::Perl::Constants> module:
 
     use Net::SSH::Perl::Constants qw( :msg );
 
@@ -753,9 +713,9 @@ each datatype that you may need to read from a packet.
 Take a look at F<eg/remoteinteract.pl> for an example of interacting
 with a remote command through the use of I<register_handler>.
 
-=item * SSH-2 Protocol
+=item * SSH2 Protocol
 
-In the SSH-2 protocol, you call I<register_handler> with two
+In the SSH2 protocol, you call I<register_handler> with two
 arguments: a string identifying the type of handler you wish to
 create, and a subroutine reference. The "string" should be, at
 this point, either C<stdout> or C<stderr>; any other string will
@@ -766,33 +726,29 @@ to handle STDERR data.
 Your subroutine reference will be passed two arguments:
 a I<Net::SSH::Perl::Channel> object that represents the open
 channel on which the data was sent, and a I<Net::SSH::Perl::Buffer>
-object containing data read from the server. In addition to these
-two arguments, the callback will be passed any additional
-arguments I<@args> that you passed to I<register_handler>; this
-can be used to give your callback functions to otherwise private
-variables, if desired.
+object containing data read from the server.
 
-This illustrates the two main differences between the SSH-1 and
-SSH-2 implementations. The first difference is that, as mentioned
+This illustrates the two main differences between the SSH1 and
+SSH2 implementations. The first difference is that, as mentioned
 above, all communication between server and client is done through
 channels, which are built on top of the main connection between
 client and server. Multiple channels are multiplexed over the
-same connection. The second difference is that, in SSH-1, you are
-processing the actual packets as they come in; in SSH-2, the packets
+same connection. The second difference is that, in SSH1, you are
+processing the actual packets as they come in; in SSH2, the packets
 have already been processed somewhat, and their contents stored in
 buffers--you are processing those buffers.
 
 The above example (the I<I received this> example) of using
-I<register_handler> in SSH-1 would look like this in SSH-2:
+I<register_handler> in SSH1 would look like this in SSH2:
 
     $ssh->register_handler("stdout", sub {
         my($channel, $buffer) = @_;
         print "I received this: ", $buffer->bytes;
     });
 
-As you can see, it's quite similar to the form used in SSH-1,
+As you can see, it's quite similar to the form used in SSH1,
 but with a few important differences, due to the differences
-mentioned above between SSH-1 and SSH-2.
+mentioned above between SSH1 and SSH2.
 
 =back
 
@@ -860,29 +816,18 @@ for more details.
 For samples/tutorials, take a look at the scripts in F<eg/> in
 the distribution directory.
 
-There is a mailing list for development discussion and usage
-questions.  Posting is limited to subscribers only.  You can sign up
-at http://lists.sourceforge.net/lists/listinfo/ssh-sftp-perl-users
+If you have any questions, code samples, bug reports, or
+feedback, please email them to:
 
-Please report all bugs via rt.cpan.org at
-https://rt.cpan.org/NoAuth/ReportBug.html?Queue=net%3A%3Assh%3A%3Aperl
+    ben@rhumba.pair.com
 
-=head1 AUTHOR
+=head1 AUTHOR & COPYRIGHT
 
-Current maintainer is David Robins, dbrobins@cpan.org.
+Benjamin Trott, ben@rhumba.pair.com
 
-Previous maintainer was Dave Rolsky, autarch@urth.org.
-
-Originally written by Benjamin Trott.
-
-=head1 COPYRIGHT
-
-Copyright (c) 2001-2003 Benjamin Trott, Copyright (c) 2003-2008 David
-Rolsky.  Copyright (c) David Robins.  All rights reserved.  This
-program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
-
-The full text of the license can be found in the LICENSE file included
-with this module.
+Except where otherwise noted, Net::SSH::Perl is Copyright
+2001 Benjamin Trott. All rights reserved. Net::SSH::Perl is
+free software; you may redistribute it and/or modify it under
+the same terms as Perl itself.
 
 =cut
